@@ -22,7 +22,7 @@
 #include <qslider.h>
 #include <qcheckbox.h>
 #include <qcombobox.h>
-#include <QtGui/qmessagebox.h>
+#include <qmessagebox.h>
 #include <qfiledialog.h>
 #include <Qt>
 #include <qbuffer.h>
@@ -31,7 +31,7 @@
 #include <QFileInfo>
 #include <QFile>
 #include <QDir>
-#include <QtGui/QHeaderView>
+#include <QHeaderView>
 
 #define TWO_BYTE_MAX 65535
 #define FOUR_CHAR 4
@@ -63,34 +63,34 @@ ControlWidget::ControlWidget(const QString &ruleFile, Control *parent)
 
     bus.registerObject("/", script, QDBusConnection::ExportAllSlots);
 
-    connect(ui->hsSignalQuality, SIGNAL(valueChanged(int)), this, SLOT(sendSQ()));
-    connect(ui->hsBatteryCharge, SIGNAL(valueChanged(int)), this, SLOT(sendBC()));
-    connect(ui->hsBatteryCharging, SIGNAL(stateChanged(int)), this, SLOT(chargingChanged(int)));
-    connect(ui->pbSelectOperator, SIGNAL(clicked()), this, SLOT(sendOPS()));
-    connect(ui->pbRegistration, SIGNAL(clicked()), this, SLOT(sendREG()));
-    connect(ui->pbSendCellBroadcast, SIGNAL(clicked()), this, SLOT(sendCBM()));
-    connect(ui->pbSendSMSMessage, SIGNAL(clicked()), this, SLOT(sendSMSMessage()));
-    connect(ui->pbFile, SIGNAL(clicked()), this, SLOT(selectFile()));
-    connect(ui->pbSendSMSDatagram, SIGNAL(clicked()), this, SLOT(sendSMSDatagram()));
-    connect(ui->pbIncomingCall, SIGNAL(clicked()), this, SLOT(sendCall()));
-    connect(ui->openSpecButton, SIGNAL(clicked()), this, SLOT(resetTranslator()));
-    connect(ui->atCheckBox, SIGNAL(clicked()), this, SLOT(atChanged()));
-    connect(ui->pbAddMessage, SIGNAL(clicked()), this, SLOT(addVoicemail()));
-    connect(ui->pbRemoveMessage, SIGNAL(clicked()), this, SLOT(delVoicemail()));
-    connect(ui->pbNotifyUDH, SIGNAL(clicked()), this, SLOT(sendVMNotify()));
-    connect(ui->pbNotifyUDHEnhanced, SIGNAL(clicked()), this, SLOT(sendEVMNotify()));
-    connect(ui->pbSendUSSD, SIGNAL(clicked()), this, SLOT(sendUSSD()));
-    connect(ui->pbCancelUSSD, SIGNAL(clicked()), this, SLOT(cancelUSSD()));
-    connect(ui->cbSimInserted, SIGNAL(clicked()), this, SLOT(simInsertRemove()));
-    connect(ui->pbStart, SIGNAL(clicked()), this, SLOT(simAppStart()));
-    connect(ui->pbAbort, SIGNAL(clicked()), this, SLOT(simAppAbort()));
-    connect(ui->pbReset, SIGNAL(clicked()), this, SLOT(modemSilentReset()));
-    connect(ui->pbSendGNSSData, SIGNAL(clicked()), this, SLOT(sendGNSSData()));
-    connect(ui->pbGNSSDefault, SIGNAL(clicked()), this, SLOT(setDefaultGNSSData()));
-    connect(ui->pbSendNotif, SIGNAL(clicked()), this, SLOT(sendCSSN()));
-    connect(ui->pbAlerting, SIGNAL(clicked()), this, SLOT(setStateAlerting()));
-    connect(ui->pbActive, SIGNAL(clicked()), this, SLOT(setStateConnected()));
-    connect(ui->pbHangup, SIGNAL(clicked()), this, SLOT(setStateHangup()));
+    connect(ui->hsSignalQuality, &QSlider::valueChanged, this, &ControlWidget::sendSQ);
+    connect(ui->hsBatteryCharge, &QSlider::valueChanged, this, &ControlWidget::sendBC);
+    connect(ui->hsBatteryCharging, &QCheckBox::stateChanged, this, &ControlWidget::chargingChanged);
+    connect(ui->pbSelectOperator, &QPushButton::clicked, this, &ControlWidget::sendOPS);
+    connect(ui->pbRegistration, &QPushButton::clicked, this, &ControlWidget::sendREG);
+    connect(ui->pbSendCellBroadcast, &QPushButton::clicked, this, &ControlWidget::sendCBM);
+    connect(ui->pbSendSMSMessage, &QPushButton::clicked, this, &ControlWidget::sendSMSMessage);
+    connect(ui->pbFile, &QPushButton::clicked, this, &ControlWidget::selectFile);
+    connect(ui->pbSendSMSDatagram, &QPushButton::clicked, this, &ControlWidget::sendSMSDatagram);
+    connect(ui->pbIncomingCall, &QPushButton::clicked, this, &ControlWidget::sendCall);
+    connect(ui->openSpecButton, &QPushButton::clicked, this, &ControlWidget::resetTranslator);
+    connect(ui->atCheckBox, &QCheckBox::clicked, this, &ControlWidget::atChanged);
+    connect(ui->pbAddMessage, &QPushButton::clicked, this, &ControlWidget::addVoicemail);
+    connect(ui->pbRemoveMessage, &QPushButton::clicked, this, &ControlWidget::delVoicemail);
+    connect(ui->pbNotifyUDH, &QPushButton::clicked, this, &ControlWidget::sendVMNotify);
+    connect(ui->pbNotifyUDHEnhanced, &QPushButton::clicked, this, &ControlWidget::sendEVMNotify);
+    connect(ui->pbSendUSSD, &QPushButton::clicked, this, &ControlWidget::sendUSSD);
+    connect(ui->pbCancelUSSD, &QPushButton::clicked, this, &ControlWidget::cancelUSSD);
+    connect(ui->cbSimInserted, &QCheckBox::clicked, this, &ControlWidget::simInsertRemove);
+    connect(ui->pbStart, &QPushButton::clicked, this, &ControlWidget::simAppStart);
+    connect(ui->pbAbort, &QPushButton::clicked, this, &ControlWidget::simAppAbort);
+    connect(ui->pbReset, &QPushButton::clicked, this, &ControlWidget::modemSilentReset);
+    connect(ui->pbSendGNSSData, &QPushButton::clicked, this, &ControlWidget::sendGNSSData);
+    connect(ui->pbGNSSDefault, &QPushButton::clicked, this, &ControlWidget::setDefaultGNSSData);
+    connect(ui->pbSendNotif, &QPushButton::clicked, this, &ControlWidget::sendCSSN);
+    connect(ui->pbAlerting, &QPushButton::clicked, this, &ControlWidget::setStateAlerting);
+    connect(ui->pbActive, &QPushButton::clicked, this, &ControlWidget::setStateConnected);
+    connect(ui->pbHangup, &QPushButton::clicked, this, &ControlWidget::setStateHangup);
 
     QStringList headers;
     headers << "Sender" << "Priority" << "Notification Status";
@@ -120,19 +120,14 @@ Control::Control(const QString& ruleFile, SimRules *sr, QObject *parent)
         : HardwareManipulator(sr, parent),
         widget(new ControlWidget(ruleFile, this))
 {
-    QList<QByteArray> proxySignals;
-    proxySignals
-        << SIGNAL(unsolicitedCommand(QString))
-        << SIGNAL(command(QString))
-        << SIGNAL(variableChanged(QString,QString))
-        << SIGNAL(switchTo(QString))
-        << SIGNAL(startIncomingCall(QString, QString, QString))
-        << SIGNAL(stateChangedToAlerting())
-        << SIGNAL(stateChangedToConnected())
-        << SIGNAL(stateChangedToHangup(int));
-
-    foreach (QByteArray sig, proxySignals)
-        connect(widget, sig, this, sig);
+    connect(widget, &ControlWidget::unsolicitedCommand, this, &Control::unsolicitedCommand);
+    connect(widget, &ControlWidget::command, this, &Control::command);
+    connect(widget, &ControlWidget::variableChanged, this, &Control::variableChanged);
+    connect(widget, &ControlWidget::switchTo, this, &Control::switchTo);
+    connect(widget, &ControlWidget::startIncomingCall, this, &Control::startIncomingCall);
+    connect(widget, &ControlWidget::stateChangedToAlerting, this, &Control::stateChangedToAlerting);
+    connect(widget, &ControlWidget::stateChangedToConnected, this, &Control::stateChangedToConnected);
+    connect(widget, &ControlWidget::stateChangedToHangup, this, &Control::stateChangedToHangup);
 }
 
 Control::~Control()
@@ -152,7 +147,7 @@ void Control::callManagement( QList<CallInfo> *list )
 
     widget->clearCallView();
 
-    foreach( CallInfo i, *list ) {
+    for ( const CallInfo &i : qAsConst(*list) ) {
         QString param[5];
 
         if ( i.incoming && !enableCSSU )
@@ -186,15 +181,15 @@ void ControlWidget::setStateAlerting()
 
 void ControlWidget::setStateHangup()
 {
-    QList <QTableWidgetItem *> items = ui->twCallMgt->selectedItems();
+    const QList <QTableWidgetItem *> items = ui->twCallMgt->selectedItems();
 
-    foreach ( QTableWidgetItem *item, items )
+    for ( QTableWidgetItem *item : items )
     {
         int row = item->row();
         if ( row >= 0 )
         {
             QTableWidgetItem *itemCallId = ui->twCallMgt->item( row, 0 );
-            if ( itemCallId != 0 )
+            if ( itemCallId != nullptr )
             {
                 QString strid = itemCallId->text();
                 int id = strid.toInt();
@@ -329,17 +324,14 @@ void ControlWidget::sendREG()
 
 void ControlWidget::sendCBM()
 {
-    QString pdu = p->constructCBMessage(ui->leMessageCode->text(),ui->cbGeographicalScope->currentIndex(),
-                       ui->leUpdateNumber->text(),ui->leChannel->text(),ui->leScheme->text(),
-                       ui->cbLanguage->currentIndex(),ui->leNumPages->text(),ui->lePage->text(),
-                       ui->teContent->toPlainText());
-
-    emit unsolicitedCommand(QString("+CBM: ")+QString::number(pdu.length()/2)+'\r'+'\n'+ pdu);
+    p->constructCBMessage(ui->leMessageCode->text(),ui->cbGeographicalScope->currentIndex(),
+                       ui->leUpdateNumber->text(),ui->leChannel->text(),
+                       ui->cbLanguage->currentIndex(),ui->teContent->toPlainText());
 }
 
 void ControlWidget::sendSMSMessage()
 {
-    if (ui->leSMSClass->text().isEmpty()) {
+    if (ui->leMessageSender->text().isEmpty()) {
         p->warning(tr("Invalid Sender"),
                 tr("Sender must not be empty"));
         return;
@@ -368,9 +360,9 @@ void ControlWidget::setDefaultGNSSData()
 
 void ControlWidget::sendGNSSData()
 {
-    QStringList xml = ui->teGNSStext->toPlainText().split("\n");
+    const QStringList xml = ui->teGNSStext->toPlainText().split("\n");
 
-    foreach ( QString line, xml )
+    for ( const QString &line : xml )
     {
         if (!line.isEmpty()) {
             QString cposr =  "+CPOSR: " + line;
@@ -476,9 +468,8 @@ void Control::handleFromData( const QString& cmd )
 
 void ControlWidget::handleToData( const QString& cmd )
 {
-    QStringList dataList = cmd.split("\n");
-    QString dataItem;
-    foreach( dataItem, dataList ){
+    const QStringList dataList = cmd.split("\n");
+    for ( auto &dataItem : dataList ){
         if( dataItem != "" ){
             ui->atViewer->append( dataItem + " : " + translator->translateCommand(dataItem) );
         }
@@ -506,7 +497,7 @@ void ControlWidget::resetTranslator()
 {
     QString fileName = QFileDialog::getOpenFileName(this,
      tr("Open Specification File"), QDir::homePath(), tr("Specification files (*.xml)"));
-    if(fileName != 0)
+    if(fileName != nullptr)
         translator->resetSpecification(fileName);
 }
 
@@ -572,7 +563,7 @@ void ControlWidget::addVoicemail()
 
 void ControlWidget::delVoicemail()
 {
-    foreach ( QTableWidgetSelectionRange range,
+    for ( QTableWidgetSelectionRange &range :
             ui->twMessageList->selectedRanges() ) {
         for ( int i = range.topRow(); i <= range.bottomRow(); i ++ ) {
             VoicemailItem &item = mailbox[i];
@@ -615,7 +606,9 @@ void ControlWidget::cancelUSSD()
 
 void ControlWidget::simInsertRemove()
 {
-    p->setSimPresent( ui->cbSimInserted->isChecked() );
+    bool isChecked = ui->cbSimInserted->isChecked();
+    p->setSimPresent( isChecked );
+    emit unsolicitedCommand( "+USIMSTATE: "+QString::number( isChecked ) );
 }
 
 void ControlWidget::handleNewApp()
@@ -647,26 +640,29 @@ void ControlWidget::modemSilentReset()
 Script::Script(QObject *obj, Ui_ControlBase *ui) : QDBusAbstractAdaptor(obj)
 {
     /* Export tabs to be accessed by script */
-    QScriptValue qsTab = engine.newQObject(ui->tab);
+    QJSValue qsTab = engine.newQObject(ui->tab);
     engine.globalObject().setProperty("tabRegistration", qsTab);
 
-    QScriptValue qsTab2 = engine.newQObject(ui->tab_2);
+    QJSValue qsTab2 = engine.newQObject(ui->tab_2);
     engine.globalObject().setProperty("tabCBM", qsTab2);
 
-    QScriptValue qsTab3 = engine.newQObject(ui->tab_3);
+    QJSValue qsTab3 = engine.newQObject(ui->tab_3);
     engine.globalObject().setProperty("tabSMS", qsTab3);
 
-    QScriptValue qsTab4 = engine.newQObject(ui->tab_4);
+    QJSValue qsTab4 = engine.newQObject(ui->tab_4);
     engine.globalObject().setProperty("tabVoiceMail", qsTab4);
 
-    QScriptValue qsTab5 = engine.newQObject(ui->tab_5);
+    QJSValue qsTab5 = engine.newQObject(ui->tab_5);
     engine.globalObject().setProperty("tabUSSD", qsTab5);
 
-    QScriptValue qsTab6 = engine.newQObject(ui->tab_6);
+    QJSValue qsTab6 = engine.newQObject(ui->tab_6);
     engine.globalObject().setProperty("tabSIM", qsTab6);
 
-    QScriptValue qsTab8 = engine.newQObject(ui->tab_8);
+    QJSValue qsTab8 = engine.newQObject(ui->tab_8);
     engine.globalObject().setProperty("tabPosition", qsTab8);
+
+    QJSValue qsTab9 = engine.newQObject(ui->tab_9);
+    engine.globalObject().setProperty("tabCall", qsTab9);
 }
 
 void Script::SetPath(const QString &path, const QDBusMessage &msg)
@@ -708,10 +704,10 @@ QString Script::Run(const QString &name, const QDBusMessage &msg)
     QString contents = stream.readAll();
     scriptFile.close();
 
-    QScriptValue qsScript = engine.evaluate(contents);
+    QJSValue qsScript = engine.evaluate(contents);
     if (qsScript.isError()) {
         QString info = fileName + ", line " + qsScript.property("lineNumber").toString() + ", " + qsScript.toString();
-        QDBusMessage reply = msg.createErrorReply(SERVICE "Error.ScriptExecError", info);
+        QDBusMessage reply = msg.createErrorReply(SERVICE ".Error.ScriptExecError", info);
         QDBusConnection::sessionBus().send(reply);
         return QString();
     }
