@@ -18,7 +18,7 @@
 ****************************************************************************/
 
 #include "simapplication.h"
-#include <qatutils.h>
+#include "qatutils.h"
 #include <qdebug.h>
 #include <QTextCodec>
 #include "qsmsmessage.h"
@@ -30,8 +30,8 @@ public:
     {
         expectedType = QSimCommand::NoCommand;
         modemHandled = false;
-        target = 0;
-        slot = 0;
+        target = nullptr;
+        slot = nullptr;
         inResponse = false;
     }
 
@@ -136,8 +136,8 @@ void SimApplication::abort()
     d->expectedType = QSimCommand::NoCommand;
     d->modemHandled = false;
     d->currentCommand = QByteArray();
-    d->target = 0;
-    d->slot = 0;
+    d->target = nullptr;
+    d->slot = nullptr;
     endSession();
 }
 
@@ -196,8 +196,8 @@ bool SimApplication::envelope( const QSimEnvelope& env )
 
     d->expectedType = QSimCommand::NoCommand;
     d->currentCommand = QByteArray();
-    d->target = 0;
-    d->slot = 0;
+    d->target = nullptr;
+    d->slot = nullptr;
     if ( env.requestHelp() )
         mainMenuHelpRequest( env.menuItem() );
     else
@@ -235,8 +235,8 @@ bool SimApplication::response( const QSimTerminalResponse& resp )
         d->modemHandled = false;
         d->currentCommand = QByteArray();
     }
-    d->target = 0;
-    d->slot = 0;
+    d->target = nullptr;
+    d->slot = nullptr;
 
     // Process the response.
     d->inResponse = true;
@@ -247,7 +247,7 @@ bool SimApplication::response( const QSimTerminalResponse& resp )
         int index = target->metaObject()->indexOfMethod( name.constData() );
         if ( index != -1 ) {
             void *args[2];
-            args[0] = 0;
+            args[0] = nullptr;
             args[1] = (void *)&resp;
             target->qt_metacall
                 ( QMetaObject::InvokeMetaMethod, index, args );
@@ -281,6 +281,11 @@ void SimApplication::endSession()
     d->rules->respond( "+CUSATEND", 1);
 }
 
+void SimApplication::reinitSim()
+{
+    d->rules->unsolicited( "+USIMSTATE: 1" );
+}
+
 DemoSimApplication::DemoSimApplication( SimRules *rules, QObject *parent )
     : SimApplication( rules, parent ), smsDestNumber( "12345" ),
     smsText( "Hello" )
@@ -383,6 +388,7 @@ const QString DemoSimApplication::getName()
 #define BIPMenu_GetChannelStatus       7
 
 #define Handled_SendSMS	1
+#define Handled_InitNAA 2
 
 enum SendSMSMenuItems {
 	SendSMS_Unpacked = 1,
@@ -494,7 +500,7 @@ void DemoSimApplication::mainMenu()
 
     cmd.setMenuItems( items );
 
-    command( cmd, 0, 0 );
+    command( cmd, nullptr, nullptr );
 }
 
 void DemoSimApplication::sendDisplayText()
@@ -956,6 +962,7 @@ void DemoSimApplication::toneMenu( const QSimTerminalResponse& resp )
     if ( resp.result() == QSimTerminalResponse::Success ) {
         // Item selected.
         cmd.setType( QSimCommand::PlayTone );
+        cmd.setText( "Play a standard supervisory tone" );
         cmd.setDestinationDevice( QSimCommand::Earpiece );
         cmd.setTone( (QSimCommand::Tone)( resp.menuItem() ) );
         if ( cmd.tone() == QSimCommand::ToneDial )
@@ -1406,6 +1413,7 @@ void DemoSimApplication::CFMenu( const QSimTerminalResponse& resp )
                 cmd.setType( QSimCommand::SendSS );
                 cmd.setDestinationDevice( QSimCommand::Network );
                 cmd.setNumber( "**62*+155543*11#" );
+                cmd.setText( "Sending SS Registration" );
                 command( cmd, this, SLOT(sendCFMenu()) );
             }
             break;
@@ -1415,6 +1423,7 @@ void DemoSimApplication::CFMenu( const QSimTerminalResponse& resp )
                 cmd.setType( QSimCommand::SendSS );
                 cmd.setDestinationDevice( QSimCommand::Network );
                 cmd.setNumber( "*62#" );
+                cmd.setText( "Sending SS Activation" );
                 command( cmd, this, SLOT(sendCFMenu()) );
             }
             break;
@@ -1424,6 +1433,7 @@ void DemoSimApplication::CFMenu( const QSimTerminalResponse& resp )
                 cmd.setType( QSimCommand::SendSS );
                 cmd.setDestinationDevice( QSimCommand::Network );
                 cmd.setNumber( "*#62**11#" );
+                cmd.setText( "Sending SS Interrogation class voice" );
                 command( cmd, this, SLOT(sendCFMenu()) );
             }
             break;
@@ -1433,6 +1443,7 @@ void DemoSimApplication::CFMenu( const QSimTerminalResponse& resp )
                 cmd.setType( QSimCommand::SendSS );
                 cmd.setDestinationDevice( QSimCommand::Network );
                 cmd.setNumber( "#62#" );
+                cmd.setText( "Sending SS Deactivation" );
                 command( cmd, this, SLOT(sendCFMenu()) );
             }
             break;
@@ -1442,6 +1453,7 @@ void DemoSimApplication::CFMenu( const QSimTerminalResponse& resp )
                 cmd.setType( QSimCommand::SendSS );
                 cmd.setDestinationDevice( QSimCommand::Network );
                 cmd.setNumber( "##62#" );
+                cmd.setText( "Sending SS Erasure" );
                 command( cmd, this, SLOT(sendCFMenu()) );
             }
             break;
@@ -2174,7 +2186,7 @@ void DemoSimApplication::timersMenuResp( const QSimTerminalResponse& resp )
         cmd.setTimerId( 2 );
         cmd.setDuration( 10000 );
         cmd.setDestinationDevice( QSimCommand::ME );
-        command( cmd, NULL, NULL );
+        command( cmd, nullptr, nullptr );
         break;
 
     case Timers_Query:
@@ -2215,7 +2227,7 @@ void DemoSimApplication::timersCmdResp( const QSimTerminalResponse& resp )
     if ( resp.command().qualifier() == 2 ) {
         if ( resp.result() == QSimTerminalResponse::Success ) {
             QString status;
-            status.sprintf( "Timer %i expires in %i:%02i:%02i.\n",
+            status.asprintf( "Timer %i expires in %i:%02i:%02i.\n",
                     resp.timerId(), resp.duration() / 3600000,
                     (resp.duration() / 60000) % 60,
                     (resp.duration() / 1000) % 60 );
@@ -2352,6 +2364,9 @@ void DemoSimApplication::refreshMenuResp( const QSimTerminalResponse& resp )
     }
 
     command( cmd, this, SLOT(endSession()) );
+
+    if (cmd.refreshType() != QSimCommand::FileChange)
+        QTimer::singleShot( 1000, this, SLOT(reinitSim()) );
 }
 
 void DemoSimApplication::sendLocalInfoMenu()
@@ -2643,6 +2658,10 @@ void DemoSimApplication::sendHandledMenu()
     item.setLabel( "Send Modem-Handled SMS" );
     items += item;
 
+    item.setIdentifier( Handled_InitNAA );
+    item.setLabel( "NAA Initialization+Full File Change Notification" );
+    items += item;
+
     cmd.setMenuItems( items );
 
     command( cmd, this, SLOT(handledMenuResp(QSimTerminalResponse)) );
@@ -2683,6 +2702,18 @@ void DemoSimApplication::handledMenuResp( const QSimTerminalResponse& resp )
         break;
     }
 
+    case Handled_InitNAA:
+    {
+        cmd.setType( QSimCommand::Refresh );
+        cmd.setQualifier( 3 );
+        cmd.setDestinationDevice( QSimCommand::ME );
+        cmd.setText( "" );
+
+        modemHandledCommand(cmd, 1000);
+        QTimer::singleShot( 1100, this, SLOT(reinitSim()) );
+        break;
+    }
+
     default:
 	endSession();
 	break;
